local _G = GLOBAL
local GI = _G.GI

GI.SaveData = function (filepath, data)
    if data and type(data) == "table" and filepath and type(filepath) == "string" then
        _G.SavePersistentString(filepath, _G.DataDumper(data, nil, true), false, nil)
    end
end
GI.LoadData = function (filepath)
    local data = nil
    _G.TheSim:GetPersistentString(filepath,
        function(load_success, str)
            if load_success == true then
                local success, savedata = _G.RunInSandboxSafe(str)
                if success and string.len(str) > 0 then
                    data = savedata
                else
                    print ("[Key Items] Could not load "..filepath)
                    return {}
                end
            else
                print ("[Key Items] Can not find "..filepath)
                return {}
            end
        end
    )
    return data or {}
end
local function Save()
    if GI.BOUNDBYNAME then -- 只有按名字绑定才允许存储
        GI.SaveData(GI.DATA_FILE, GI.DATA)
    end
end
GI.Save = Save
local function Load()
    if GI.BOUNDBYNAME then
        GI.DATA = GI.LoadData(GI.DATA_FILE)
    end
end
GI.Load = Load
local function Init(session)
    if GI.DATA == nil then GI.DATA = {} end
    if GI.DATA["DPI"] == nil then GI.DATA["DPI"] = 200 end
    if GI.DATA["ATLAS"] == nil then GI.DATA["ATLAS"] = {} end
    if GI.DATA["IMAGE"] == nil then GI.DATA["IMAGE"] = {} end
    if GI.DATA["SKIN"] == nil then GI.DATA["SKIN"] = {} end
    if GI.DATA["DisName"] == nil then GI.DATA["DisName"] = {} end
    if GI.DATA["MemoryAll"] == nil then GI.DATA["MemoryAll"] = {} end
    if session and GI.DATA[session] == nil then GI.DATA[session] = {} end
    if GI.KeyDown["TOGGLE"] == nil then GI.KeyDown["TOGGLE"] = 0 end
end
GI.Init = Init
--Utils--
local function GetIconOffset()
    local POS_R = 200.0
    if GI.DATA["DPI"] then
        POS_R = GI.DATA["DPI"]
    end
    POS_R = POS_R + 30
    local POS_X = POS_R/1.4149
    local _iconpos_z4 = 
    {
        {0, POS_R}, {POS_R, 0}, {0, -POS_R}, {-POS_R, 0},
    }
    local _iconpos_x4 =
    {
        [5] = {POS_X, POS_X}, [6] = {POS_X, -POS_X}, [7] = {-POS_X, -POS_X}, [8] = {-POS_X, POS_X},
    }
    local _iconpos_8 = 
    {
        {0, POS_R}, {POS_R, 0}, {0, -POS_R}, {-POS_R, 0}, {POS_X, POS_X}, {POS_X, -POS_X}, {-POS_X, -POS_X}, {-POS_X, POS_X}
    }
    GI._iconpos = _iconpos_8
    if GI.STYLE == "+" then GI._iconpos = _iconpos_z4 end
    if GI.STYLE == "x" then GI._iconpos = _iconpos_x4 end
    return GI._iconpos
end
GI.GetIconOffset = GetIconOffset
local function GetR(dx, dy)
    local r = 0
    r = math.sqrt(math.pow(dx, 2) + math.pow(dy, 2))
    return r
end
GI.GetR = GetR
local function GetKeyIndex(dx, dy, huge)
    local dpi = 200
    if GI.DATA and GI.DATA["DPI"] ~= nil then
        dpi = GI.DATA["DPI"]
    end
    if GI.GetR(dx, dy) <= dpi then
        if GI.STYLE == "+" or GI.STYLE == "8" then return -1 end
        if GI.STYLE == "x" then return -1 end
    end
    --local a = 0
    --local b = 0
    --if STYLE == "+" then a = 1 b = 4 end
    --if STYLE == "x" then a = 5 b = 8 end
    --if STYLE == "8" then a = 1 b = 8 end
    local min = 8000
    local index = 0
    --for i=a,b do
    for k,v in pairs((not huge) and GI.GetIconOffset() or GI.GetIconPos()) do
        local temp = GetR(v[1] - dx, v[2] - dy)
        if min > temp then
            index = k
            min = temp
        end
    end
    return index
end
GI.GetKeyIndex = GetKeyIndex
local function CanBuild(player, prefab)
    if prefab and player.replica.builder:KnowsRecipe(prefab) and player.replica.builder:CanBuild(prefab) then
        return true
    end
    return false
end
GI.CanBuild = CanBuild
local function UpdateOldIndex(item, keyindex)
    if item and item.replica.equippable ~= nil then
        if item.replica.equippable:EquipSlot() == _G.EQUIPSLOTS.HANDS then
            GI.old_index = GI.now_index or 0
            GI.now_index = keyindex
        end
    end
end
GI.UpdateOldIndex = UpdateOldIndex
local function TryUseItem(item, keyindex, exchange, nouse)
    local inv = _G.ThePlayer.replica.inventory
    if inv == nil then return end
    local items = inv:GetItems()
    for k,v in pairs(items) do
        if v and v.prefab == item then
            if exchange then
                GI.UpdateOldIndex(v, keyindex)
            end
            if not nouse then
                inv:UseItemFromInvTile(v)
            end
            return true
        end
    end
    local container = inv:GetOverflowContainer()
    if  container ~= nil then
        if container.slots == nil then
            items = container:GetItems()
        else
            items = container.slots
        end

        for k,v in pairs(items) do
            if v and v.prefab == item then
                if exchange then
                    GI.UpdateOldIndex(v, keyindex)
                end
                if not nouse then
                    inv:UseItemFromInvTile(v)
                end
                return true
            end
        end
    end
    items = inv:GetEquips()
    if items ~= nil then
        for k,v in pairs(items) do
            if v and v.prefab == item then
                if exchange then
                    GI.UpdateOldIndex(v, keyindex)
                end
                if not nouse then
                    inv:UseItemFromInvTile(v)
                end
                return true
            end
        end
    end
    return false
end
GI.TryUseItem = TryUseItem
local function GetAtlasAndImg(item)
    if item and type(item) == "table" and item.replica.inventoryitem then
        local atlas = item.replica.inventoryitem:GetAtlas()
        if atlas and type(atlas) == "string" and atlas == "images/inventoryimages.xml" then
            atlas = nil
        end
        return atlas or nil,item.replica.inventoryitem:GetImage() or nil
    end
end
GI.GetAtlasAndImg = GetAtlasAndImg
local function IsHUDScreen()
    local defaultscreen = false
    if _G.TheFrontEnd:GetActiveScreen() and _G.TheFrontEnd:GetActiveScreen().name and type(_G.TheFrontEnd:GetActiveScreen().name) == "string" and _G.TheFrontEnd:GetActiveScreen().name == "HUD" then
        defaultscreen = true
    end
    return defaultscreen
end
GI.IsHUDScreen = IsHUDScreen
local function MoveCursorToCenter( force)
    if force or GI.SHOWINCENTER then
        _G.TheInputProxy:SetOSCursorPos(GI.gim:GetCursorCenter())
    end
end
GI.MoveCursorToCenter = MoveCursorToCenter

local function UpdataCursor(x0, y0, t)
    local fin = true
    local x , y = _G.TheInputProxy:GetOSCursorPos()
    if GI.breakFlag then 
        --print("breakFlag") 
        GI.gim:UpdateWithCursor(x0, y0, t, fin) 
        return 
    end
    _G.ThePlayer:DoTaskInTime(.1, function ()    
        if x and y then
            local dx = x-x0 local dy = y-y0
            --print("x,y_x0,y0=",x,y,x0,y0)
            local index = GI.GetKeyIndex(dx, dy)
            GI.index = index
            GI.r = GI.GetR(dx, dy)
            GI.gim:UpdateWithCursor(x0, y0, t)
        end
        GI.UpdataCursor(x0, y0, t)
    end)
end
GI.UpdataCursor = UpdataCursor







GI.MOUSE_RIGHT = 1001 
GI.MOUSE_MIDDLE = 1002
local function MouseButtonFn()
    if not GI.IsHUDScreen() then return end
    if _G.TheWorld == nil then return end

    if _G.TheInput:IsMouseDown(GI.MOUSE_MIDDLE) and not GI.KeyDown["M_MIDDLE"] then 
        GI.KeyDown["M_MIDDLE"] = true
        if _G.TheInput:IsKeyDown(GI.BOUNDKEY) then
            GI.KeyDown["TOGGLE"] = 2
            MoveCursorToCenter()
            GI.BoundDown()    
        else
            GI.KeyDown["TOGGLE"] = 1
            MoveCursorToCenter()
            GI.ToggleDown()
        end
        return
    end
    if not _G.TheInput:IsMouseDown(GI.MOUSE_MIDDLE) and GI.KeyDown["M_MIDDLE"] then
        if GI.KeyDown["TOGGLE"] == 2 then
            GI.KeyDown["TOGGLE"] = 0
            GI.BoundUp()
        elseif GI.KeyDown["TOGGLE"] == 1 then
            GI.KeyDown["TOGGLE"] = 0
            GI.ToggleUp()
        end
        GI.KeyDown["M_MIDDLE"] = false
        return
    end
    if _G.TheInput:IsMouseDown(GI.MOUSE_RIGHT) and GI.KeyDown["TOGGLE"] > 0 then
        GI.KeyDown["M_RIGHT"] = true
        GI.MouseRightDown()
        return
    end
    if not _G.TheInput:IsMouseDown(GI.MOUSE_RIGHT) and GI.KeyDown["M_RIGHT"] then
        GI.MouseRightUp()
        GI.KeyDown["M_RIGHT"] = false
        return
    end
end
GI.MouseButtonFn = MouseButtonFn
local function ToggleKeyDownFn()
    if not GI.IsHUDScreen() then return end
    if _G.TheWorld == nil then return end

    if GI.KeyDown["TOGGLE"] and GI.KeyDown["TOGGLE"] > 0 then return end
    if _G.TheInput:IsKeyDown(GI.BOUNDKEY) then
        GI.KeyDown["TOGGLE"] = 2
        MoveCursorToCenter()
        GI.BoundDown()
    else
        GI.KeyDown["TOGGLE"] = 1
        MoveCursorToCenter()
        GI.ToggleDown()
    end
end
GI.ToggleKeyDownFn = ToggleKeyDownFn
local function ToggleKeyUpFn()
    if not GI.KeyDown["TOGGLE"] or GI.KeyDown["TOGGLE"] <= 0 then return end
    if GI.KeyDown["TOGGLE"] == 1 then
        GI.ToggleUp()
        GI.KeyDown["TOGGLE"] = 0
    elseif GI.KeyDown["TOGGLE"] == 2 then
        GI.BoundUp()
        GI.KeyDown["TOGGLE"] = 0
    end
end
GI.ToggleKeyUpFn = ToggleKeyUpFn
local function DPIKeyDownFn()
    if not GI.IsHUDScreen() then return end
    if _G.TheWorld == nil then return end

    if GI.KeyDown["DPI"] then return end
    GI.KeyDown["DPI"] = true
    MoveCursorToCenter(GI.FORCE)
    GI.DPIDown()
end
GI.DPIKeyDownFn = DPIKeyDownFn
local function DPIKeyUpFn()
    if not GI.KeyDown["DPI"] then return end
    GI.DPIUp()
    GI.KeyDown["DPI"] = false
end
GI.DPIKeyUpFn = DPIKeyUpFn