
local Widget = require "widgets/widget" --Widget，所有widget的祖先类
local Text = require "widgets/text" --Text类，文本处理
local Image = require "widgets/image"

local GIMain = Class(Widget, function(self)
    Widget._ctor(self, "GIMain")
    GI.gim = self

    self.itemroot = self:AddChild(Widget("itemroot"))
    self.dpiroot = self:AddChild(Widget("dpiroot"))

    self.centxet = self.dpiroot:AddChild(Text(BODYTEXTFONT, 36,"＋")) --添加一个文本变量，接收Text实例。
    --self.text = self:AddChild(Badge(BODYTEXTFONT, 100,"＋"))
    self.yuandpi = self.dpiroot:AddChild(Image("images/yuan.xml", "yuan.tex", nil))
    self.yuandpi_old = self:AddChild(Image("images/yuan.xml", "yuan.tex", nil))

    self:HideAll()

    self.itempos = {}
    self.itemicon = {}
    

end)
function GIMain:HideAll()
    local gim = self
    if gim.itemroot.shown then gim.itemroot:Hide() end
    if gim.dpiroot.shown then gim.dpiroot:Hide() end
    if gim.yuandpi_old.shown then gim.yuandpi_old:Hide() end
end
function GIMain:ShowOldYuan()
    local dpi = 200
    if GI and GI.DATA and GI.DATA["DPI"] then 
        dpi = GI.DATA["DPI"] or 200
    end
    self.yuandpi_old:SetScale(dpi/375, dpi/375, 1)
    self.yuandpi_old:SetTint(0,0,0,1)
    self.yuandpi_old:MoveToBack()
    self.yuandpi_old:Show()
end
function GIMain:GetCursorCenter()
    local screenwidth, screenheight = TheSim:GetScreenSize()
    local cx = math.floor(screenwidth/2 + 0.5)
    local cy = math.floor(screenheight/2 + 0.5)
    return cx, cy
end
function GIMain:SetPositionTo(x, y)
    local cx, cy = self:GetCursorCenter()
    self:SetPosition(x-cx, y-cy, 0)
end

function GIMain:UpdateWithCursor(x0, y0, t, fin)
    local gim = GI.gim
    if fin then gim:HideAll() return end
    if type(t) == "string" and t == "DPI" then
        if gim.breakCheck then return end
        local x , y = TheInputProxy:GetOSCursorPos()
        if x and y then
            local r = GI.GetR(x-x0, y-y0)
            if r > 500 then r = 500 end
            gim.yuandpi:SetScale(r/375,r/375,1)
        end
        gim:SetPositionTo(x0, y0)
        if not gim.dpiroot.shown then gim.dpiroot:Show() end
        if not gim.yuandpi.shown then gim.yuandpi:Show() end
        if not gim.yuandpi_old.shown then gim:ShowOldYuan() end
    end
    if type(t) == "string" and t == "toggle" then
        if gim.breakCheck then return end
        local x , y = TheInputProxy:GetOSCursorPos()
        if not x or not y then return end
        local index = GI.index or 0
        for _,v in pairs(gim.itemicon) do
            v:SetScale(.7,.7,1)
        end
        if index > 0 and gim.itemicon[index] then
            gim.itemicon[index]:SetScale(1.4,1.4,1)
            gim.itemicon[index]:MoveToFront()
        end
        if not gim.itemroot.shown then gim.itemroot:Show() end
    end
    if type(t) == "string" and t == "bound" then
        if gim.breakCheck then return end
        local x , y = TheInputProxy:GetOSCursorPos()
        if not x or not y then return end
        local index = GI.index or 0
        for _,v in pairs(gim.itemicon) do
            v:SetScale(.7,.7,1)
        end
        if index > 0 and gim.itemicon[index] then
            gim.itemicon[index]:SetScale(1.4,1.4,1)
            gim.itemicon[index]:MoveToFront()
        end
        if not gim.itemroot.shown then gim.itemroot:Show() end
        if not gim.yuandpi_old.shown then gim:ShowOldYuan() end
    end
end


function GIMain:ShowFromCursor(x, y, noshow)
    if noshow then self.itemroot:Hide() return end
    self:SetPositionTo(x, y)
    self:UpdateItemIcon(GI.DATA[GI.SESSION])
    self:Show()
    self.itemroot:Show()
end
function GIMain:UpdateItemIcon(itemnames)
    self.itemroot:MoveToFront()
    if type(itemnames) ~= "table" then return end
    local namemaxn = table.maxn(itemnames)
    local iconmaxn = table.maxn(self.itemicon)
    for i=1, namemaxn >= iconmaxn and namemaxn or iconmaxn do 
        local v = itemnames[i]
        local atlas = GI.DATA["ATLAS"][v] or "images/inventoryimages.xml"
        local image = GI.DATA["IMAGE"][v]
        local canUse = GI.TryUseItem(v, i, not GI.EXCHANGE, GI.NOUSE)
        local canb = GI.CanBuild(ThePlayer, v)
        if self.itemicon[i] ~= nil then
            if v == nil or atlas == nil or image == nil then
                --print("Hide",v)
                self.itemicon[i]:Hide()
            else
                self.itemicon[i]:SetTexture(atlas, image, nil)
                self.itemicon[i]:SetPosition(GI.GetIconOffset()[i][1], GI.GetIconOffset()[i][2], 0)
                self.itemicon[i]:SetHoverText(GI.DATA["DisName"][v])
                if not canUse then
                    if not canb then
                        self.itemicon[i]:SetTint(0,0,0,1)
                    else
                        self.itemicon[i]:SetTint(80/256,80/256,80/256,1)
                    end
                else
                    self.itemicon[i]:SetTint(1,1,1,1)
                end
                self.itemicon[i]:SetScale(.7,.7,1)
                self.itemicon[i]:Show()
            end
        else
            if v and atlas ~= nil and image ~= nil then
                self.itemicon[i] = self.itemroot:AddChild(Image(atlas, image, nil))
                self.itemicon[i]:SetPosition(GI.GetIconOffset()[i][1],GI.GetIconOffset()[i][2],0)
                self.itemicon[i]:SetHoverText(GI.DATA["DisName"][v])
                self.itemicon[i]:SetScale(.7,.7,1)
                if not canUse then
                    if not canb then
                        self.itemicon[i]:SetTint(0,0,0,1)
                    else
                        self.itemicon[i]:SetTint(80/256,80/256,80/256,1)
                    end
                else
                    self.itemicon[i]:SetTint(1,1,1,1)
                end
            end
        end
    end
end
return GIMain
